@@ -1,3 +1,21 @@
+#
+# Copyright The NOMAD Authors.
+#
+# This file is part of NOMAD.
+# See https://nomad-lab.eu for further info.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 import numpy as np
 import os
 import logging
@@ -7,10 +25,9 @@ import pint
 from .metainfo import m_env
 from nomad.parsing.parser import FairdiParser
 
-from nomad.parsing.file_parser import Quantity, UnstructuredTextFileParser
-from nomad.datamodel.metainfo.public import section_run, section_sampling_method,\
-    section_system, section_single_configuration_calculation, section_energy_contribution,\
-    Workflow, MolecularDynamics
+from nomad.parsing.file_parser import Quantity, TextParser
+from nomad.datamodel.metainfo.common_dft import Run, SamplingMethod, System,\
+    SingleConfigurationCalculation, EnergyContribution, Workflow, MolecularDynamics
 from nomad.datamodel.metainfo.common import section_topology, section_interaction
 from .metainfo.lammps import x_lammps_section_input_output_files, x_lammps_section_control_parameters
 
@@ -85,7 +102,7 @@ def get_unit(units_type, property_type=None, dimension=3):
         return units
 
 
-class DataParser(UnstructuredTextFileParser):
+class DataParser(TextParser):
     def __init__(self):
         self._headers = [
             'atoms', 'bonds', 'angles', 'dihedrals', 'impropers', 'atom types', 'bond types',
@@ -109,8 +126,8 @@ class DataParser(UnstructuredTextFileParser):
     def init_quantities(self):
         self._quantities = []
         for header in self._headers:
-            self._quantities.append(
-                Quantity(header, r'\s*([\+\-eE\d\. ]+)\s*%s\s*\n' % header, comment='#'))
+            self._quantities.append(Quantity(
+                header, r'\s*([\+\-eE\d\. ]+)\s*%s\s*\n' % header, comment='#', repeats=True))
 
         def get_section_value(val):
             val = val.split('\n')
@@ -137,7 +154,7 @@ class DataParser(UnstructuredTextFileParser):
             self._quantities.append(
                 Quantity(
                     section, r'\s*%s\s*(#*\s*[\s\S]*?\n)\n*([\deE\-\+\.\s]+)\n' % section,
-                    str_operation=get_section_value))
+                    str_operation=get_section_value, repeats=True))
 
     def get_interactions(self):
         styles_coeffs = []
@@ -153,7 +170,7 @@ class DataParser(UnstructuredTextFileParser):
         return styles_coeffs
 
 
-class TrajParser(UnstructuredTextFileParser):
+class TrajParser(TextParser):
     def __init__(self):
         self._masses = None
         self._reference_masses = dict(
@@ -183,15 +200,17 @@ class TrajParser(UnstructuredTextFileParser):
 
         self._quantities = [
             Quantity(
-                'time_step', r'\s*ITEM:\s*TIMESTEP\s*\n\s*(\d+)\s*\n', comment='#'),
+                'time_step', r'\s*ITEM:\s*TIMESTEP\s*\n\s*(\d+)\s*\n', comment='#',
+                repeats=True),
             Quantity(
-                'n_atoms', r'\s*ITEM:\s*NUMBER OF ATOMS\s*\n\s*(\d+)\s*\n', comment='#'),
+                'n_atoms', r'\s*ITEM:\s*NUMBER OF ATOMS\s*\n\s*(\d+)\s*\n', comment='#',
+                repeats=True),
             Quantity(
                 'pbc_cell', r'\s*ITEM: BOX BOUNDS\s*([\s\w]+)([\+\-\d\.eE\s]+)\n',
-                str_operation=get_pbc_cell, comment='#'),
+                str_operation=get_pbc_cell, comment='#', repeats=True),
             Quantity(
                 'atoms_info', r's*ITEM:\s*ATOMS\s*([ \w]+\n)*?([\+\-eE\d\.\n ]+)\n*I*',
-                str_operation=get_atoms_info, comment='#')
+                str_operation=get_atoms_info, comment='#', repeats=True)
         ]
 
     def with_trajectory(self):
@@ -291,7 +310,7 @@ class TrajParser(UnstructuredTextFileParser):
         return pint.Quantity(forces, self._units.get('force', None))
 
 
-class LogParser(UnstructuredTextFileParser):
+class LogParser(TextParser):
     def __init__(self):
         self._commands = [
             'angle_coeff', 'angle_style', 'atom_modify', 'atom_style', 'balance',
@@ -329,25 +348,18 @@ class LogParser(UnstructuredTextFileParser):
         self._quantities = [
             Quantity(
                 name, r'\n\s*%s\s+([\w\. \/\#\-]+)(\&\n[\w\. \/\#\-]*)*' % name,
-                str_operation=str_op, comment='#') for name in self._commands]
+                str_operation=str_op, comment='#', repeats=True) for name in self._commands]
 
-        self._quantities.append(
-            Quantity('program_version', r'\s*LAMMPS\s*\(([\w ]+)\)\n', dtype=str)
+        self._quantities.append(Quantity(
+            'program_version', r'\s*LAMMPS\s*\(([\w ]+)\)\n', dtype=str, repeats=False,
+            flatten=False)
         )
 
-        self._quantities.append(
-            Quantity('finished', r'\s*Dangerous builds\s*=\s*(\d+)')
+        self._quantities.append(Quantity(
+            'finished', r'\s*Dangerous builds\s*=\s*(\d+)', repeats=False)
         )
 
-    @property
-    def units(self):
-        if self._file_handler is None or self._units is None:
-            units_type = self.get('units', ['lj'])[0]
-            self._units = get_unit(units_type)
-        return self._units
-
-    def get_thermodynamic_data(self):
-        def str_operation(val):
+        def str_to_thermo(val):
             res = {}
             if val.count('Step') > 1:
                 val = val.replace('-', '').replace('=', '').replace('(sec)', '').split()
@@ -369,14 +381,23 @@ class LogParser(UnstructuredTextFileParser):
 
             return res
 
-        pattern = r'\s*\-*(\s*Step\s*[\-\s\w\.\=\(\)]*[ \-\.\d\n]+)Loop'
+        self._quantities.append(Quantity(
+            'thermo_data', r'\s*\-*(\s*Step\s*[\-\s\w\.\=\(\)]*[ \-\.\d\n]+)Loop',
+            str_operation=str_to_thermo, repeats=False, convert=False)
+        )
 
-        parser = UnstructuredTextFileParser(
-            self.mainfile, [Quantity('thermo_data', pattern, str_operation=str_operation)])
+    @property
+    def units(self):
+        if self._file_handler is None or self._units is None:
+            units_type = self.get('units', ['lj'])[0]
+            self._units = get_unit(units_type)
+        return self._units
 
-        thermo_data = parser['thermo_data']
+    def get_thermodynamic_data(self):
+        self._thermo_data = self.get('thermo_data')
 
-        self._thermo_data = list(thermo_data)[0] if thermo_data is not None else thermo_data
+        if self._thermo_data is None:
+            return
 
         for key, val in self._thermo_data.items():
             low_key = key.lower()
@@ -393,7 +414,8 @@ class LogParser(UnstructuredTextFileParser):
         dump = self.get('dump')
         if dump is None:
             self.logger.warn(
-                'Trajectory not specified in %s, will scan directory' % self.maindir)
+                'Trajectory not specified in directory, will scan.',
+                data=dict(directory=self.maindir))
             traj_files = os.listdir(self.maindir)
             traj_files = [f for f in traj_files if f.endswith('trj')]
 
@@ -409,7 +431,8 @@ class LogParser(UnstructuredTextFileParser):
         read_data = self.get('read_data')
         if read_data is None:
             self.logger.warn(
-                'Data file not specified in %s, will scan directory' % self.maindir)
+                'Data file not specified in directory, will scan.',
+                data=dict(directory=self.maindir))
             data_files = os.listdir(self.maindir)
             data_files = [f for f in data_files if f.endswith('data') or f.startswith('data')]
 
@@ -422,10 +445,6 @@ class LogParser(UnstructuredTextFileParser):
         pbc = self.get('boundary', ['p', 'p', 'p'])
         return [v == 'p' for v in pbc]
 
-    def get_program_version(self):
-        version = self.get('program_version', [''])[0]
-        return ' '.join(version)
-
     def get_sampling_method(self):
         fix_style = self.get('fix', [[''] * 3])[0][2]
 
@@ -478,7 +497,7 @@ class LogParser(UnstructuredTextFileParser):
                 pass
 
         else:
-            self.logger.warn('Fix style %s not supported' % fix_style)
+            self.logger.warn('Fix style not supported', data=dict(style=fix_style))
 
         return res
 
@@ -505,12 +524,6 @@ class LogParser(UnstructuredTextFileParser):
 
         return styles_coeffs
 
-    def finished_normally(self):
-        return self.get('finished') is not None
-
-    def with_thermodynamics(self):
-        return self._thermo_data is not None
-
 
 class LammpsParser(FairdiParser):
     def __init__(self):
@@ -544,23 +557,23 @@ class LammpsParser(FairdiParser):
         if sec_sccs:
             if len(sec_sccs) != n_evaluations:
                 self.logger.warn(
-                    '''Mismatch in number of calculations %d and number of property
-                    evaluations %d!, will create new sections''' % (
-                        len(sec_sccs), n_evaluations))
+                    '''Mismatch in number of calculations and number of property
+                    evaluations!, will create new sections''',
+                    data=dict(n_calculations=len(sec_sccs), n_evaluations=n_evaluations))
 
             else:
                 create_scc = False
 
         for n in range(n_evaluations):
             if create_scc:
-                sec_scc = sec_run.m_create(section_single_configuration_calculation)
+                sec_scc = sec_run.m_create(SingleConfigurationCalculation)
             else:
                 sec_scc = sec_sccs[n]
 
             for key, val in thermo_data.items():
                 key = key.lower()
                 if key in energy_keys_mapping:
-                    sec_energy = sec_scc.m_create(section_energy_contribution)
+                    sec_energy = sec_scc.m_create(EnergyContribution)
                     sec_energy.energy_contibution_kind = energy_keys_mapping[key]
                     sec_energy.energy_contribution_value = val[n]
 
@@ -582,11 +595,12 @@ class LammpsParser(FairdiParser):
                 else:
                     if n == 0:
                         self.logger.warn(
-                            'Unsupported property %s in thermodynamic data' % key)
+                            'Unsupported property in thermodynamic data',
+                            data=dict(property=key))
 
     def parse_sampling_method(self):
         sec_run = self.archive.section_run[-1]
-        sec_sampling_method = sec_run.m_create(section_sampling_method)
+        sec_sampling_method = sec_run.m_create(SamplingMethod)
 
         run_style = self.log_parser.get('run_style', ['verlet'])[0]
         run = self.log_parser.get('run', [0])[0]
@@ -625,14 +639,17 @@ class LammpsParser(FairdiParser):
         pbc_cell = self.traj_parser.get('pbc_cell', [])
         n_atoms = self.traj_parser.get('n_atoms', [])
 
+        if len(n_atoms) == 0:
+            return
+
         create_scc = True
         sec_sccs = sec_run.section_single_configuration_calculation
         if sec_sccs:
             if len(sec_sccs) != len(pbc_cell):
                 self.logger.warn(
-                    '''Mismatch in number of calculations %d and number of property
-                    evaluations %d!, will create new sections''' % (
-                        len(sec_sccs), len(pbc_cell)))
+                    '''Mismatch in number of calculations and number of structures,
+                    will create new sections''',
+                    data=dict(n_calculations=len(sec_sccs), n_structures=len(pbc_cell)))
 
             else:
                 create_scc = False
@@ -641,7 +658,7 @@ class LammpsParser(FairdiParser):
         self.traj_parser._units = self.log_parser.units
 
         for i in range(len(pbc_cell)):
-            sec_system = sec_run.m_create(section_system)
+            sec_system = sec_run.m_create(System)
             sec_system.number_of_atoms = n_atoms[i]
             sec_system.configuration_periodic_dimensions = pbc_cell[i][0]
             sec_system.simulation_cell = pbc_cell[i][1] * distance_unit
@@ -658,7 +675,7 @@ class LammpsParser(FairdiParser):
             forces = self.traj_parser.get_forces(i)
             if forces is not None:
                 if create_scc:
-                    sec_scc = sec_run.m_create(section_single_configuration_calculation)
+                    sec_scc = sec_run.m_create(SingleConfigurationCalculation)
                 else:
                     sec_scc = sec_sccs[i]
 
@@ -667,6 +684,9 @@ class LammpsParser(FairdiParser):
     def parse_topology(self):
         sec_run = self.archive.section_run[-1]
 
+        if self.traj_parser.mainfile is None or self.data_parser.mainfile is None:
+            return
+
         masses = self.data_parser.get('Masses', None)
 
         self.traj_parser.masses = masses
@@ -710,24 +730,29 @@ class LammpsParser(FairdiParser):
                     val = ' '.join([str(v) for v in val])
                 setattr(sec_control_parameters, key, str(val))
 
-    def _init_parsers(self):
+    def init_parser(self):
         self.log_parser.mainfile = self.filepath
         self.log_parser.logger = self.logger
         self.traj_parser.logger = self.logger
         self.data_parser.logger = self.logger
 
+    def reuse_parser(self, parser):
+        self.log_parser.quantities = parser.log_parser.quantities
+        self.traj_parser.quantities = parser.traj_parser.quantities
+        self.data_parser.quantities = parser.data_parser.quantities
+
     def parse(self, filepath, archive, logger):
         self.filepath = filepath
         self.archive = archive
         self.logger = logger if logger is not None else logging
 
-        self._init_parsers()
+        self.init_parser()
 
-        sec_run = self.archive.m_create(section_run)
+        sec_run = self.archive.m_create(Run)
 
         # parse basic
         sec_run.program_name = 'LAMMPS'
-        sec_run.program_version = self.log_parser.get_program_version()
+        sec_run.program_version = self.log_parser.get('program_version', '')
 
         # parse method-related
         self.parse_sampling_method()
@@ -760,6 +785,6 @@ class LammpsParser(FairdiParser):
             if sec_workflow.workflow_type == 'molecular_dynamics':
                 sec_md = sec_workflow.m_create(MolecularDynamics)
 
-                sec_md.finished_normally = self.log_parser.finished_normally()
+                sec_md.finished_normally = self.log_parser.get('finished') is not None
                 sec_md.with_trajectory = self.traj_parser.with_trajectory()
-                sec_md.with_thermodynamics = self.log_parser.with_thermodynamics()
+                sec_md.with_thermodynamics = self.log_parser.get('thermo_data') is not None
