@@ -7,16 +7,6 @@ import re
 import ast
 from collections import namedtuple
 
-import metainfo
-
-COMMON_META_INFO_PATH = os.path.normpath(os.path.join(
-    os.path.dirname(os.path.abspath(metainfo.__file__)),
-    "common.nomadmetainfo.json"))
-
-PUBLIC_META_INFO_PATH = os.path.normpath(os.path.join(
-    os.path.dirname(os.path.abspath(metainfo.__file__)),
-    "public.nomadmetainfo.json"))
-
 NOTEXCEPT = re.compile(r'[a-cf-zA-CF-Z!\?,{}\[\]]')
 
 def is_number(val):
@@ -756,25 +746,8 @@ class JsonMetaInfo(object):
         Loads data and extracts values of items
         with specified superNames
     """
-    def __init__(self, *args):
-        self.jsonList = None
-        for filepath in args:
-            try:
-                with open(filepath, encoding="utf-8") as f:
-                    jsonDict = json.load(f)
-            except:
-                logging.exception("Error while loading file %s" % filepath)
-                raise
-            typeStr = jsonDict.get("type","nomad_meta_info_1_0")
-            typeRe = re.compile(r"nomad_meta_info_(?P<major>[0-9]+)_(?P<minor>[0-9]+)$")
-            m = typeRe.match(typeStr)
-            if not m:
-                raise Exception("unexpected type '%s', expected nomad_meta_info_1_0" % typeStr)
-            newJsonList = jsonDict.get("metaInfos",[])
-            if self.jsonList:
-                self.jsonList = self.jsonList + newJsonList
-            else:
-                self.jsonList = newJsonList
+    def __init__(self, metainfo_env):
+        self.jsonList = metainfo_env.to_legacy_dict(metainfo_env.packages)['metaInfos']
 
     def attributes(self, sectionname):
         attributes = {}
@@ -889,7 +862,8 @@ if __name__ == "__main__":
             'section_repository_info'
             ]}
 
-    jsonmetadata = JsonMetaInfo(COMMON_META_INFO_PATH, PUBLIC_META_INFO_PATH)
+    from nomad.datamodel.metainfo import m_env
+    jsonmetadata = JsonMetaInfo(m_env)
 
     updateDict = {
             'startSection' : [['section_topology']],
