@@ -41,7 +41,7 @@ def qa(skip_tests: bool, exitfirst: bool):
     click.echo('Run code style checks ...')
     ret_code += os.system('python -m pycodestyle --ignore=E501,E701,E731 nomad tests')
     click.echo('Run linter ...')
-    ret_code += os.system('python -m pylint --load-plugins=pylint_mongoengine,nomad/metainfo/pylint_plugin nomad tests')
+    ret_code += os.system('python -m pylint --load-plugins=pylint_mongoengine,nomad.metainfo.pylint_plugin nomad tests')
     click.echo('Run static type checks ...')
     ret_code += os.system('python -m mypy --ignore-missing-imports --follow-imports=silent --no-strict-optional nomad tests')
 
@@ -194,15 +194,13 @@ def toolkit_metadata():
 @dev.command(help=(
     'Updates parser`s README files by combining a general template with  '
     'a parser`s metadata YAML file.'))
-def update_parsers_readmes():
+@click.option('--parser', help='Only updated the README of the given parsers subdirctory.')
+def update_parser_readmes(parser):
     from glob import glob
     import re
     import yaml
-    print('\nPWD: ', os.getcwd())
-    print('WARNING: to be run from project`s root dir\n')
 
-    # based on
-    # https://www.kite.com/python/answers/how-to-update-and-replace-text-in-a-file-in-python
+    os.chdir(os.path.join(os.path.dirname(__file__), '../..'))
 
     # filenames
     local_fn = 'README.md'
@@ -210,7 +208,11 @@ def update_parsers_readmes():
     parser_path = './dependencies/parsers/'
 
     for num, ddir in enumerate(sorted(glob(parser_path + '*/')), start=1):
-        parser_dirname = ddir.split('/')[-2]
+        if parser is not None and parser != ddir.split(os.sep)[-2]:
+            print(f'Skip {ddir}')
+            continue
+
+        _, parser_dirname = os.path.split(ddir)
         print('{} Working on {}' .format(num, parser_dirname))
 
         # Open general template
@@ -221,6 +223,10 @@ def update_parsers_readmes():
         local_mdata = ddir + 'metadata.yaml'
         with open(local_mdata, 'r') as mdata_f:
             mdata = yaml.load(mdata_f, Loader=yaml.FullLoader)
+            if mdata.get('codeName', '').strip() == '':
+                mdata['codeName'] = os.path.basename(os.path.dirname(ddir))
+            if 'preamble' not in mdata:
+                mdata['preamble'] = ''
 
         # Find & Replace Parser`s metadata on its README file
         local_readme = ddir + local_fn
@@ -230,14 +236,14 @@ def update_parsers_readmes():
                 if key in ignore:
                     continue
 
-                print('\tReplacing', key)
                 find = r'\$' + key + r'\$'
                 replace = mdata[key]
+                print(f'\tReplacing {key} with {replace}')
 
                 if key == 'parserSpecific':
                     if mdata[key] != '':
-                        replace = r'---\n## Parser Specific\n' + replace
-                # print('\t', find, ' -> ', replace)
+                        replace = r'## Parser Specific\n' + replace
+
                 body = re.sub(find, replace, body)
 
             # Extra: to replace the codeName (there's no YAML key for this)
@@ -259,57 +265,7 @@ def update_parsers_readmes():
 
             # save file
             local.seek(0)  # go to the top
-            local.write(body)
-            local.truncate()
-
-
-@dev.command(help='Updates parser`s metadata YAML')
-def update_parsers_metadata():
-    import re
-    from glob import glob
-    print('\nPWD: ', os.getcwd())
-    print('WARNING: to be run from project`s root dir\n')
-
-    # Based on
-    # https://www.kite.com/python/answers/how-to-update-and-replace-text-in-a-file-in-python
-
-    # dictionary to change YAML keys to camelCase
-    new_keys = {
-        'code-label': 'codeLabel',
-        'code-label-style': 'codeLabelStyle',
-        'code-url': 'codeUrl',
-        'parser-dir-name': 'parserDirName',
-        'parser-git-url': 'parserGitUrl',
-        'parser-specific': 'parserSpecific',
-        'table-of-files': 'tableOfFiles'}
-
-    num = 0
-    parser_path = './dependencies/parsers/'
-    for ddir in sorted(glob(parser_path + '*/')):
-        num += 1
-        parser_dirname = ddir.split('/')[-2]
-        print('{} Working on {}' .format(num, parser_dirname))
-
-        local_mdata = ddir + 'metadata.yaml'
-        if os.path.isfile(local_mdata) is False:
-            print('\tWARNING: old readme file absent: {}'.format(local_mdata))
-            continue
-
-        # Replace key names in metadata.yaml
-        # we open it as plain text: YAML module is unnecesary
-        with open(local_mdata, 'r+') as local:
-            body = local.read()
-
-            for key in new_keys.keys():
-                # Add  ':' to ensure complete sentence
-                find, replace = key + ':', new_keys[key] + ':'
-
-                # print('\t {:15} \t -> {} ' .format(find, replace))
-                body = re.sub(find, replace, body)
-
-            # save file
-            local.seek(0)  # go to the top
-            local.write(body)
+            local.write(body.strip())
             local.truncate()
 
 
