@@ -15,6 +15,7 @@
 
 import logging
 import mmap
+import io
 import re
 import numpy as np
 import pint
@@ -35,7 +36,7 @@ class ParsePattern:
                 token += r'Ee\+\d\.\-'
             if 'int' in value:
                 token += r'\d'
-            if 'string' in value:
+            if 'str' in value:
                 token += r'\w'
             if 'array' in value:
                 token += r' '
@@ -73,14 +74,14 @@ class ParsePattern:
 
 class Quantity:
     '''
-    Class to define a quantity to be parsed in the UnstructuredTextFileParser.
+    Class to define a quantity to be parsed in the TextParser.
 
     Arguments:
         quantity: string to identify the name or a metainfo quantity to initialize the
             quantity object.
         re_pattern: pattern to be used by re for matching. Ideally, overlaps among
             quantities for a given parser should be avoided.
-        sub_parser: instance of UnstructuredTextFileParser to perform local parsing
+        sub_parser: instance of TextParser to perform local parsing
             within a matched block
         str_operation: external function to be performed on a matched block
         dtype: data type of the quantity
@@ -103,7 +104,7 @@ class Quantity:
             self.shape = None
         elif isinstance(quantity, mQuantity):
             self.name = quantity.name
-            self.dtype = quantity.type
+            self.dtype = quantity.type.type if isinstance(quantity.type, np.dtype) else quantity.type
             self.unit = quantity.unit
             # check if metainfo shape has dependencies
             self.shape = quantity.shape
@@ -117,9 +118,10 @@ class Quantity:
         self._re_pattern: str = re_pattern.re_pattern if isinstance(
             re_pattern, ParsePattern) else re_pattern
         self._str_operation: Callable = kwargs.get('str_operation', None)
-        self._sub_parser: UnstructuredTextFileParser = kwargs.get('sub_parser', None)
-        self.repeats: bool = kwargs.get('repeats', True)
+        self._sub_parser: TextParser = kwargs.get('sub_parser', None)
+        self.repeats: bool = kwargs.get('repeats', False)
         self.convert: bool = kwargs.get('convert', True)
+        self.flatten: bool = kwargs.get('flatten', True)
         self.comment: str = kwargs.get('comment', None)
 
     @property
@@ -156,7 +158,7 @@ class Quantity:
             if self.str_operation is not None:
                 val = self.str_operation(val)
 
-            else:
+            elif self.flatten:
                 val = val.strip().split() if isinstance(val, str) else val
                 val = val[0] if len(val) == 1 else val
 
@@ -170,8 +172,12 @@ class Quantity:
                                 val = float(val)
                             except Exception:
                                 pass
+                    else:
+                        try:
+                            val = self.dtype(val)
+                        except Exception:
+                            pass
 
-                    self.shape = [] if self.shape is None else self.shape
                     return val
 
                 elif type(val) in [np.ndarray, list]:
@@ -181,7 +187,6 @@ class Quantity:
                         if self.dtype is None:
                             if np.all(np.mod(val_test, 1) == 0):
                                 val_test = np.array(val_test, dtype=int)
-                        self.shape = list(np.shape(val)) if self.shape is None else self.shape
                         val = val_test
 
                     except Exception:
@@ -191,32 +196,32 @@ class Quantity:
 
                 elif isinstance(val, dict):
                     for k, v in val.items():
-                        self.dtype = None
                         val[k] = _convert(v)
                     return val
 
                 else:
-                    self.dtype = type(val)
-                    self.shape = [] if self.shape is None else self.shape
                     return val
 
             if self.convert:
                 val = _convert(val)
 
             if isinstance(val, np.ndarray) and self.shape:
-                val = np.reshape(val, self.shape)
+                try:
+                    val = np.reshape(val, self.shape)
+                except Exception:
+                    pass
 
             return val
 
         val_out = [process(val) for val in val_in]
 
         if isinstance(val_out[0], np.ndarray):
-            self.dtype = val_out[0].dtype
+            self.dtype = val_out[0].dtype  # type: ignore
 
         return val_out
 
 
-class DataTextFileParser(FileParser):
+class DataTextParser(FileParser):
     '''
     Parser for structured data text files using numpy.loadtxt
 
@@ -256,7 +261,7 @@ class DataTextFileParser(FileParser):
         return self._file_handler
 
 
-class UnstructuredTextFileParser(FileParser):
+class TextParser(FileParser):
     '''
     Parser for unstructured text files using the re module. The quantities to be parsed
     are given as a list of Quantity objects which specifies the re pattern. The mmap
@@ -287,7 +292,7 @@ class UnstructuredTextFileParser(FileParser):
         '''
         Returns a copy of the object excluding the parsed results.
         '''
-        return UnstructuredTextFileParser(
+        return TextParser(
             self.mainfile, self.quantities, self.logger, **self._kwargs)
 
     def init_quantities(self):
@@ -333,12 +338,15 @@ class UnstructuredTextFileParser(FileParser):
         Memory mapped representation of the file.
         '''
         if self._file_handler is None:
-            with open(self.mainfile) as f:
-                self._file_handler = mmap.mmap(
-                    f.fileno(), self._file_length, access=mmap.ACCESS_COPY,
-                    offset=self._file_offset)
-                # set the extra chunk loaded before the intended offset to empty
-                self._file_handler[:self._file_pad] = b' ' * self._file_pad
+            with self.open(self.mainfile) as f:
+                if isinstance(f, io.TextIOWrapper):
+                    self._file_handler = mmap.mmap(
+                        f.fileno(), self._file_length, access=mmap.ACCESS_COPY,
+                        offset=self._file_offset)
+                    # set the extra chunk loaded before the intended offset to empty
+                    self._file_handler[:self._file_pad] = b' ' * self._file_pad
+                else:
+                    self._file_handler = f.read()
             self._file_pad = 0
         return self._file_handler
 
@@ -406,8 +414,8 @@ class UnstructuredTextFileParser(FileParser):
 
                 self._results[quantities[i].name] = value_processed
 
-            except Exception as e:
-                self.logger.warn('Error setting value for %s ' % quantities[i].name, exc_info=e)
+            except Exception:
+                self.logger.warn('Error setting value', data=dict(quantity=quantities[i].name))
                 pass
 
     def _parse_quantity(self, quantity):
@@ -421,7 +429,8 @@ class UnstructuredTextFileParser(FileParser):
                     span = np.array(res.span()) + self.file_offset
                     sub_parser = quantity._sub_parser.copy()
                     sub_parser.mainfile = self.mainfile
-                    if (span[1] - span[0]) < mmap.PAGESIZE:
+                    sub_parser.logger = self.logger
+                    if (span[1] - span[0]) < mmap.PAGESIZE or True:
                         # self.logger.warn(
                         #     'Cannot use sub parser on quantity %s with blocks with size <'
                         #     '%d. Will try to parse string' % (quantity.name, mmap.PAGESIZE))
@@ -434,7 +443,7 @@ class UnstructuredTextFileParser(FileParser):
                 else:
                     unit = res.groupdict().get('__unit_%s' % quantity.name, None)
                     units.append(unit.decode() if unit is not None else None)
-                    value.append(''.join(
+                    value.append(' '.join(
                         [group.decode() for group in res.groups() if group and group != unit]))
 
         else:
@@ -443,7 +452,8 @@ class UnstructuredTextFileParser(FileParser):
                     span = np.array(res.span()) + self.file_offset
                     sub_parser = quantity._sub_parser.copy()
                     sub_parser.mainfile = self.mainfile
-                    if (span[1] - span[0]) < mmap.PAGESIZE:
+                    sub_parser.logger = self.logger
+                    if (span[1] - span[0]) < mmap.PAGESIZE or True:
                         # self.logger.warn(
                         #     'Cannot use sub parser on quantity %s with blocks with size <'
                         #     '%d. Will try to parse string' % (quantity.name, mmap.PAGESIZE))
@@ -456,7 +466,7 @@ class UnstructuredTextFileParser(FileParser):
                 else:
                     unit = res.groupdict().get('__unit_%s' % quantity.name, None)
                     value.append(
-                        ''.join([group.decode() for group in res.groups() if group and group != unit]))
+                        ' '.join([group.decode() for group in res.groups() if group and group != unit]))
                     units.append(unit.decode() if unit is not None else None)
 
         if not value:
@@ -479,7 +489,7 @@ class UnstructuredTextFileParser(FileParser):
 
                 self._results[quantity.name] = value_processed
             except Exception:
-                self.logger.warn('Error setting value for %s ' % quantity.name)
+                self.logger.warn('Error setting value', data=dict(quantity=quantity.name))
                 pass
 
     def parse(self, key=None):
